{{

  Version 101211a works resonably well.
  This object uses a modified version of
  Nick McClanahan's Magic IR.
  This will record the IR output of up to
  32 buttons from an IR remote.
  The codes are saved in EEPROM so they may
  be replayed after a power cycle.


}}
CON              
  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000

  _DebugBaud = 9600
 
OBJ

  magicir : "magicir101202e"
  Debug : "fullduplexserial"
  Eeprom : "Propeller EepromDWD"   ' no added cog, uses I2C lines.

VAR

  long  codePtr, code1len, code1[128 * 32], codeLength[32]
  byte  recordFlag[32]
  
PUB main | localCharacter, localButton

  'recordFlag := 0
   
  magicir.Start(0, @codePtr, @code1len)
  Debug.Start(31, 30, 0, _DebugBaud)
  waitcnt(clkfreq * 2 + cnt)
  DisplayMainMenu
  repeat
    
    localCharacter := Debug.rxcheck
    if localCharacter == "r"
      localButton := Record
      View(localButton)
      DisplayMainMenu
    elseif localCharacter == "p"                                           
      localButton := Play
      View(localButton)
      DisplayMainMenu
    elseif localCharacter == "v"
      localButton := EnterButtonId                                        
      View(localButton)
      DisplayMainMenu
    elseif localCharacter == "c"                                           
      localButton := EnterButtonId
      Clear(localButton)
      DisplayMainMenu
    elseif localCharacter == "d"                                           
      ViewAll
      DisplayMainMenu
    
   'waitcnt(clkfreq/10 + cnt)

PUB Record : localButton | localCharacter

  localButton := -1
  Debug.str(string(13, "Enter button number you wish to record."))
  repeat while localButton == -1
    localButton := EnterButtonId
  Debug.str(string(13, "Recording IR code."))
  codePtr := @code1 + (128 * 4 * localButton)
  magicir.storecode'(codePtr)
  if code1len < 1
    Debug.str(string(13, "No data recorded."))
    Debug.str(string(13, "Press any key to try again."))
    localCharacter := Debug.rx
    Debug.str(string(13, "Recording IR code."))
    magicir.storecode'(localButton)
    if code1len < 1
      Debug.str(string(13, "Still no data recorded."))
      Debug.str(string(13, "Back to main menu."))
      recordFlag[localButton] := 0
    else
      Debug.str(string(13, "Successful recording."))    
      recordFlag[localButton] := 1
      codeLength[localButton] := code1len
      'eeprom.FromRam(codePtr, codePtr + (code1len * 4) - 1, codePtr)
      'eeprom.FromRam(@codeLength + (localButton * 4) , @codeLength + (localButton * 4) + 3, @codeLength + (localButton * 4))
      'eeprom.FromRam(@recordFlag + (localButton * 4) , @recordFlag + (localButton * 4) + 3, @recordFlag + (localButton * 4))
  else
    Debug.str(string(13, "Successful recording."))    
    recordFlag[localButton] := 1
    codeLength[localButton] := code1len
    'eeprom.FromRam(codePtr, codePtr + (code1len * 4) - 1, codePtr)
    'eeprom.FromRam(@codeLength + (localButton * 4) , @codeLength + (localButton * 4) + 3, @codeLength + (localButton * 4))
  eeprom.FromRam(codePtr, codePtr + (code1len * 4) - 1, codePtr)
  eeprom.FromRam(@codeLength + (localButton * 4) , @codeLength + (localButton * 4) + 3, @codeLength + (localButton * 4))
  eeprom.FromRam(@recordFlag + (localButton * 4) , @recordFlag + (localButton * 4) + 3, @recordFlag + (localButton * 4))    
  code1len := 0
    
PUB Play : localButton 

  localButton := -1
  repeat while localButton == -1
    localButton := EnterButtonId

  if recordFlag[localButton] == 1 
    Debug.str(string(13, "Playing IR code."))
    repeat 10 '3
      magicir.playcode(1, @code1 + (128 * 4 * localButton) , codeLength[localButton])
  else
    Debug.str(string(13, "No IR code recorded yet."))   

PUB View(localButton)

  if localButton == -1
    Debug.str(string(13, "No button information provided."))
    return
  if codeLength[localButton] < 1
    Debug.str(string(13, "No code recorded for button "))
    Debug.dec(localButton)
    Debug.str(string(".", 13))
    return
  Debug.str(string(13, "Length of IR code #"))
  Debug.dec(localButton)
  Debug.str(string(" = "))
  Debug.dec(codeLength[localButton])
  Debug.str(string(13, "IR code # "))
  Debug.dec(localButton)
  Debug.str(string(" = ", 13))
  DumpBuffer(@code1 + (128 * 4 * localButton) , codeLength[localButton])
  
PUB DumpBuffer(localPointer, localSize) | localIndex

  localIndex := 0
  Debug.str(string(13, "long  "))
  repeat localIndex from 0 to localSize - 1
    debug.tx(36)
    debug.hex(long[localPointer], 8)
    localPointer += 4
    if localIndex // 8 == 7
      Debug.str(string(13, "long  "))        
    elseif localIndex <> localSize - 1
      debug.tx(44)            
      debug.tx(32)
  debug.tx(13)
   
PUB clear(localButton) '| localIndex

  if localButton == -1
    Debug.str(string(13, "No button information provided."))
    return

  longfill(@code1 + (128 * 4 * localButton), 0, 128)
  Debug.str(string(13, "Buffer # "))
  Debug.dec(localButton)
  Debug.str(string(" Cleared"))
  codeLength[localButton] := 0
  debug.tx(13)
   
PUB EnterButtonId | localCharacter, localIdFlag, localIdValue

  
  result := -1
  Debug.str(string("Enter button number", 13))  
  Debug.str(string("(Followed by Enter key)", 13)) 
  localIdValue := 0
  localIdFlag := 0
  localCharacter := 0
  repeat while localCharacter <> 13
    localCharacter := Debug.rx
    if localCharacter => "0" and localCharacter =< "9"
      localIdValue := (localIdValue * 10) +  (localCharacter - 48)
      localIdFlag := 1
  if localIdFlag == 0
    Debug.str(string(13, "No button number selected.", 13))
  elseif localIdValue > 31 or localIdValue < 0
    Debug.str(string("Button number out of range.  0 to 31 are valid options.", 13))
  else
    result := localIdValue
    Debug.str(string("Button # is "))
    Debug.dec(localIdValue)
    Debug.tx(13)

PUB ViewAll | localIndex

  Debug.str(string(13, "All IR codes listed below. "))
  repeat localIndex from 0 to 31
    View(localIndex)

PUB DisplayMainMenu

  Debug.str(string(13, "Press r to record IR code "))
  Debug.str(string(13, "Press p to play IR code "))
  Debug.str(string(13, "Press v to view IR code "))
  Debug.str(string(13, "Press c to clear buffer"))
  Debug.str(string(13, "Press d to display all IR codes"))
    